@@ -1,5 +1,5 @@
 /* sort - sort lines of text (with all kinds of options).
-   Copyright (C) 1988, 1991 Free Software Foundation
+   Copyright (C) 1988, 1991, 1992, 1993, 1994 Free Software Foundation
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
    The author may be reached (Email) at the address mike@gnu.ai.mit.edu,
    or (US mail) as Mike Haertel c/o Free Software Foundation. */
 
+#include <config.h>
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
@@ -26,6 +28,10 @@
 #include <signal.h>
 #include <stdio.h>
 #include "system.h"
+#include "version.h"
+#include "long-options.h"
+#include "safe-stat.h"
+
 #ifdef _POSIX_VERSION
 #include <limits.h>
 #else
@@ -1447,6 +1453,9 @@ main (argc, argv)
 #endif				/* _POSIX_VERSION */
 
   program_name = argv[0];
+
+  parse_long_options (argc, argv, "sort", version_string, usage);
+
   have_read_stdin = 0;
   inittables ();
 
@@ -1528,7 +1537,7 @@ main (argc, argv)
 	  if (digits[UCHAR (*s)])
 	    {
 	      if (!key)
-		usage ();
+		usage (2);
 	      for (t = 0; digits[UCHAR (*s)]; ++s)
 		t = t * 10 + *s - '0';
 	      t2 = 0;
@@ -1651,21 +1660,26 @@ main (argc, argv)
 		  case 'T':
 		    if (s[1])
 		      temp_file_prefix = ++s;
-		    else if (i < argc - 1)
-		      {
-			temp_file_prefix = argv[++i];
-			goto outer;
-		      }
 		    else
-		      error (2, 0, "option `-T' requires an argument");
+		      {
+			if (i < argc - 1)
+			  temp_file_prefix = argv[++i];
+			else
+			  error (2, 0, "option `-T' requires an argument");
+		      }
+		    goto outer;
 		    break;
 		  case 'u':
 		    unique = 1;
 		    break;
+		  case 'y':
+		    /* Accept and ignore e.g. -y0 for compatibility with
+		       Solaris 2.  */
+		    goto outer;
 		  default:
 		    fprintf (stderr, "%s: unrecognized option `-%c'\n",
 			     argv[0], *s);
-		    usage ();
+		    usage (2);
 		  }
 		if (*s)
 		  ++s;
@@ -1711,32 +1725,61 @@ main (argc, argv)
 
   if (strcmp (outfile, "-"))
     {
-      for (i = 0; i < nfiles; ++i)
-	if (!strcmp (outfile, files[i]))
-	  break;
-      if (i == nfiles)
-	ofp = xfopen (outfile, "w");
-      else
+      struct stat outstat;
+      if (SAFE_STAT (outfile, &outstat) == 0)
 	{
-	  char buf[8192];
-	  FILE *fp = xfopen (outfile, "r");
-	  int cc;
+	  /* The following code prevents a race condition when
+	     people use the brain dead shell programming idiom:
+		  cat file | sort -o file
+	     This feature is provided for historical compatibility,
+	     but we strongly discourage ever relying on this in
+	     new shell programs. */
 
-	  tmp = tempname ();
-	  ofp = xfopen (tmp, "w");
-	  while ((cc = fread (buf, 1, sizeof buf, fp)) > 0)
-	    xfwrite (buf, 1, cc, ofp);
-	  if (ferror (fp))
+	  /* Temporarily copy each input file that might be another name
+	     for the output file.  When in doubt (e.g. a pipe), copy.  */
+	  for (i = 0; i < nfiles; ++i)
 	    {
-	      error (0, errno, "%s", outfile);
-	      cleanup ();
-	      exit (2);
+	      char buf[8192];
+	      FILE *fp;
+	      int cc;
+
+	      if (S_ISREG (outstat.st_mode) && strcmp (outfile, files[i]))
+		{
+		  struct stat instat;
+		  if ((strcmp (files[i], "-")
+		       ? SAFE_STAT (files[i], &instat)
+		       : fstat (fileno (stdin), &instat)) != 0)
+		    {
+		      error (0, errno, "%s", files[i]);
+		      cleanup ();
+		      exit (2);
+		    }
+		  if (S_ISREG (instat.st_mode)
+		      && (instat.st_ino != outstat.st_ino
+			  || instat.st_dev != outstat.st_dev))
+		    {
+		      /* We know the files are distinct.  */
+		      continue;
+		    }
+		}
+
+	      fp = xfopen (files[i], "r");
+	      tmp = tempname ();
+	      ofp = xfopen (tmp, "w");
+	      while ((cc = fread (buf, 1, sizeof buf, fp)) > 0)
+		xfwrite (buf, 1, cc, ofp);
+	      if (ferror (fp))
+		{
+		  error (0, errno, "%s", files[i]);
+		  cleanup ();
+		  exit (2);
+		}
+	      xfclose (ofp);
+	      xfclose (fp);
+	      files[i] = tmp;
 	    }
-	  xfclose (ofp);
-	  xfclose (fp);
-	  files[i] = tmp;
-	  ofp = xfopen (outfile, "w");
 	}
+      ofp = xfopen (outfile, "w");
     }
   else
     ofp = stdout;
@@ -1764,11 +1807,46 @@ main (argc, argv)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-cmus] [-t separator] [-o output-file] [-T tempdir] [-bdfiMnr]\n\
-       [+POS1 [-POS2]] [-k POS1[,POS2]] [file...]\n",
-	   program_name);
-  exit (2);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  +POS1 [-POS2]    start a key at POS1, end it before POS2\n\
+  -M               compare (unknown) < `JAN' < ... < `DEC', imply -b\n\
+  -T DIRECT        use DIRECT for temporary files, not $TMPDIR or /tmp\n\
+  -b               ignore leading blanks in sort fields or keys\n\
+  -c               check if given files already sorted, do not sort\n\
+  -d               consider only [a-zA-Z0-9 ] characters in keys\n\
+  -f               fold lower case to upper case characters in keys\n\
+  -i               consider only [\\040-\\0176] characters in keys\n\
+  -k POS1[,POS2]   same as +POS1 [-POS2], but all positions counted from 1\n\
+  -m               merge already sorted files, do not sort\n\
+  -n               compare according to string numerical value, imply -b\n\
+  -o FILE          write result on FILE instead of standard output\n\
+  -r               reverse the result of comparisons\n\
+  -s               stabilize sort by disabling last resort comparison\n\
+  -t SEP           use SEParator instead of non- to whitespace transition\n\
+  -u               with -c, check for strict ordering\n\
+  -u               with -m, only output the first of an equal sequence\n\
+      --help       display this help and exit\n\
+      --version    output version information and exit\n\
+\n\
+POS is F[.C][OPTS], where F is the field number and C the character\n\
+position in the field, both counted from zero.  OPTS is made up of one\n\
+or more of Mbdfinr, this effectively disable global -Mbdfinr settings\n\
+for that key.  If no key given, use the entire line as key.  With no\n\
+FILE, or when FILE is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
