@@ -17,16 +17,7 @@
 
 /* Written by Jim Meyering, meyering@cs.utexas.edu.  */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
 
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
@@ -201,6 +192,7 @@ struct Spec_list
 char *xmalloc ();
 char *stpcpy ();
 void error ();
+int safe_read ();
 
 /* The name by which this program was run.  */
 char *program_name;
@@ -342,7 +334,7 @@ Usage: %s [OPTION]... SET1 [SET2]\n\
 SETs are specified as strings of characters.  Most represent\n\
 themselves.  Here are the special writings:\n\
 \n\
-  \\NNN            character with octal value NNN (1 to 3 digits)\n\
+  \\NNN            character with octal value NNN (1 to 3 octal digits)\n\
   \\\\              backslash\n\
   \\a              audible BEL\n\
   \\b              backspace\n\
@@ -361,11 +353,11 @@ themselves.  Here are the special writings:\n\
   [:cntrl:]       all control characters\n\
   [:digit:]       all digits\n\
   [:graph:]       all printable characters, not including space\n\
-  [:lower:]       all lowercase letters\n\
+  [:lower:]       all lower case letters\n\
   [:print:]       all printable characters, including space\n\
   [:punct:]       all punctuation characters\n\
   [:space:]       all horizontal or vertical whitespace\n\
-  [:upper:]       all uppercase letters\n\
+  [:upper:]       all upper case letters\n\
   [:xdigit:]      all hexadecimal digits\n\
   [=CHAR=]        all characters which are equivalent to CHAR\n\
 ");
@@ -447,14 +439,13 @@ is_char_class_member (char_class, c)
     }
 }
 
-/* Perform the first pass over each range-spec argument S,
-   converting all \c and \ddd escapes to their one-byte representations.
-   The conversion is done in-place, so S must point to writable
-   storage.  If an illegal quote sequence is found, an error message is
-   printed and the function returns non-zero.  Otherwise the length of
-   the resulting string is returned through LEN and the function returns 0.
-   The resulting array of characters may contain zero-bytes; however,
-   on input, S is assumed to be null-terminated, and hence
+/* Perform the first pass over each range-spec argument S, converting all
+   \c and \ddd escapes to their one-byte representations.  The conversion
+   is done in-place, so S must point to writable storage.  If an invalid
+   quote sequence is found print an error message and return non-zero.
+   Otherwise set *LEN to the length of the resulting string and return
+   zero.  The resulting array of characters may contain zero-bytes;
+   however, on input, S is assumed to be null-terminated, and hence
    cannot contain actual (non-escaped) zero bytes.  */
 
 static int
@@ -604,7 +595,7 @@ make_printable_char (c)
 
 static char *
 make_printable_str (s, len)
-     unsigned char *s;
+     const unsigned char *s;
      int len;
 {
   /* Worst case is that every character expands to a backslash
@@ -718,7 +709,7 @@ append_range (list, first, last)
 /* If CHAR_CLASS_STR is a valid character class string, append a
    newly allocated structure representing that character class to the end
    of the specification list LIST and return 0.  If CHAR_CLASS_STR is not
-   a valid string, give an error message and return non-zero.  */
+   a valid string, print an error message and return non-zero.  */
 
 static int
 append_char_class (list, char_class_str, len)
@@ -749,9 +740,9 @@ append_char_class (list, char_class_str, len)
 }
 
 /* Append a newly allocated structure representing a [c*n]
-   repeated character construct, to the specification list LIST.
+   repeated character construct to the specification list LIST.
    THE_CHAR is the single character to be repeated, and REPEAT_COUNT
-   is non-negative repeat count.  */
+   is a non-negative repeat count.  */
 
 static void
 append_repeated_char (list, the_char, repeat_count)
@@ -804,16 +795,16 @@ append_equiv_class (list, equiv_class_str, len)
   return 0;
 }
 
-/* Return a newly allocated copy of P[FIRST_IDX..LAST_IDX].  */
+/* Return a newly allocated copy of the substring P[FIRST_IDX..LAST_IDX].  */
 
 static unsigned char *
 substr (p, first_idx, last_idx)
-     unsigned char *p;
+     const unsigned char *p;
      int first_idx;
      int last_idx;
 {
   int len = last_idx - first_idx + 1;
-  unsigned char *tmp = (unsigned char *) xmalloc (len);
+  unsigned char *tmp = (unsigned char *) xmalloc (len + 1);
 
   assert (first_idx <= last_idx);
   /* We must use bcopy or memcpy rather than strncpy
@@ -830,7 +821,7 @@ substr (p, first_idx, last_idx)
 
 static int
 find_closing_delim (p, start_idx, p_len, pre_bracket_char)
-     unsigned char *p;
+     const unsigned char *p;
      int start_idx;
      int p_len;
      unsigned int pre_bracket_char;
@@ -852,7 +843,7 @@ find_closing_delim (p, start_idx, p_len, pre_bracket_char)
 
 static int
 non_neg_strtol (s, len, val)
-     unsigned char *s;
+     const unsigned char *s;
      int len;
      long int *val;
 {
@@ -896,7 +887,7 @@ non_neg_strtol (s, len, val)
 
 static int
 find_bracketed_repeat (p, start_idx, p_len, char_to_repeat, n)
-     unsigned char *p;
+     const unsigned char *p;
      int start_idx;
      int p_len;
      unsigned int *char_to_repeat;
@@ -912,7 +903,7 @@ find_bracketed_repeat (p, start_idx, p_len, char_to_repeat, n)
     {
       if (p[i] == ']')
 	{
-	  unsigned char *digit_str;
+	  const unsigned char *digit_str;
 	  int digit_str_len = i - start_idx - 2;
 
 	  *char_to_repeat = p[start_idx];
@@ -953,11 +944,11 @@ find_bracketed_repeat (p, start_idx, p_len, char_to_repeat, n)
 
 static int
 build_spec_list (unescaped_string, len, result)
-     unsigned char *unescaped_string;
+     const unsigned char *unescaped_string;
      int len;
      struct Spec_list *result;
 {
-  unsigned char *p;
+  const unsigned char *p;
   int i;
 
   p = unescaped_string;
@@ -988,7 +979,8 @@ build_spec_list (unescaped_string, len, result)
 	      if (closing_delim_idx >= 0)
 		{
 		  int parse_failed;
-		  unsigned char *opnd_str = substr (p, i + 2, closing_delim_idx - 1);
+		  unsigned char *opnd_str = substr (p, i + 2,
+						    closing_delim_idx - 1);
 		  if (p[i + 1] == ':')
 		    parse_failed = append_char_class (result, opnd_str,
 				     (closing_delim_idx - 1) - (i + 2) + 1);
@@ -1007,7 +999,7 @@ build_spec_list (unescaped_string, len, result)
 	      /* Else fall through.  This could be [:*] or [=*].  */
 	    default:
 	      /* Determine whether this is a bracketed repeat range
-		 matching the RE \[.\*(dec_or_oct_number)?\].  */
+	         matching the RE \[.\*(dec_or_oct_number)?\].  */
 	      closing_bracket_idx = find_bracketed_repeat (p, i + 1,
 				       len, &char_to_repeat, &repeat_count);
 	      if (closing_bracket_idx >= 0)
@@ -1153,9 +1145,9 @@ get_next (s, class)
 
     case RE_EQUIV_CLASS:
       /* FIXME: this assumes that each character is alone in its own
-	 equivalence class (which appears to be correct for my
-	 LC_COLLATE.  But I don't know of any function that allows
-	 one to determine a character's equivalence class.  */
+         equivalence class (which appears to be correct for my
+         LC_COLLATE.  But I don't know of any function that allows
+         one to determine a character's equivalence class.  */
 
       return_val = p->u.equiv_code;
       s->state = NEW_ELEMENT;
@@ -1358,7 +1350,7 @@ parse_str (s, spec_list)
 
 static void
 string2_extend (s1, s2)
-     struct Spec_list *s1;
+     const struct Spec_list *s1;
      struct Spec_list *s2;
 {
   struct List_element *p;
@@ -1392,7 +1384,7 @@ string2_extend (s1, s2)
 
     case RE_EQUIV_CLASS:
       /* This shouldn't happen, because validate exits with an error
-	 if it finds an equiv class in string2 when translating.  */
+         if it finds an equiv class in string2 when translating.  */
       abort ();
       break;
 
@@ -1420,7 +1412,7 @@ string2_extend (s1, s2)
 
 static void
 validate (s1, s2)
-     struct Spec_list *s1;
+     const struct Spec_list *s1;
      struct Spec_list *s2;
 {
   get_spec_stats (s1, -1);
@@ -1429,13 +1421,14 @@ validate (s1, s2)
       error (1, 0, "the [c*] repeat construct may not appear in string1");
     }
 
-  /* FIXME: it isn't clear from the POSIX spec that this is illegal,
+  /* FIXME: it isn't clear from the POSIX spec that this is invalid,
      but in the spirit of the other restrictions put on translation
      with character classes, this seems a logical interpretation.  */
   if (complement && s1->has_upper_or_lower)
     {
       error (1, 0,
-	     "character classes may not be used when translating and complementing");
+	     "character classes may not be used when translating \
+and complementing");
     }
 
   if (s2)
@@ -1444,8 +1437,8 @@ validate (s1, s2)
       if (s2->has_restricted_char_class)
 	{
 	  error (1, 0,
-		 "when translating, the only character classes that may appear in\n\
-\tstring2 are `upper' and `lower'");
+		 "when translating, the only character classes that may \
+appear in\n\tstring2 are `upper' and `lower'");
 	}
 
       if (s2->n_indefinite_repeats > 1)
@@ -1458,7 +1451,8 @@ validate (s1, s2)
 	  if (s2->has_equiv_class)
 	    {
 	      error (1, 0,
-		     "[=c=] expressions may not appear in string2 when translating");
+		     "[=c=] expressions may not appear in string2 \
+when translating");
 	    }
 
 	  if (s1->length > s2->length)
@@ -1477,14 +1471,16 @@ validate (s1, s2)
 
 	  if (complement && s2->has_upper_or_lower)
 	    error (1, 0,
-		   "character classes may not be used when translating and complementing");
+		   "character classes may not be used when translating \
+and complementing");
 	}
       else
 	/* Not translating.  */
 	{
 	  if (s2->n_indefinite_repeats > 0)
 	    error (1, 0,
-		   "the [c*] construct may appear in string2 only when translating");
+		   "the [c*] construct may appear in string2 only \
+when translating");
 	}
     }
 }
@@ -1514,7 +1510,7 @@ squeeze_filter (buf, size, reader)
       if (i >= nr)
 	{
 	  if (reader == NULL)
-	    nr = read (0, (char *) buf, size);
+	    nr = safe_read (0, (char *) buf, size);
 	  else
 	    nr = (*reader) (buf, size, NULL);
 
@@ -1559,13 +1555,13 @@ squeeze_filter (buf, size, reader)
 	      out_len = i - begin + 1;
 
 	      /* But since we stepped by 2 in the loop above,
-		 out_len may be one too large.  */
+	         out_len may be one too large.  */
 	      if (i > 0 && buf[i - 1] == char_to_squeeze)
 		--out_len;
 
 	      /* Advance i to the index of first character to be
-		 considered when looking for a char different from
-		 char_to_squeeze.  */
+	         considered when looking for a char different from
+	         char_to_squeeze.  */
 	      ++i;
 	    }
 	  if (out_len > 0
@@ -1616,7 +1612,7 @@ read_and_delete (buf, size, not_used)
   do
     {
       int i;
-      int nr = read (0, (char *) buf, size);
+      int nr = safe_read (0, (char *) buf, size);
 
       if (nr < 0)
 	error (1, errno, "read error");
@@ -1664,7 +1660,7 @@ read_and_xlate (buf, size, not_used)
   if (hit_eof)
     return 0;
 
-  chars_read = read (0, (char *) buf, size);
+  chars_read = safe_read (0, (char *) buf, size);
   if (chars_read < 0)
     error (1, errno, "read error");
   if (chars_read == 0)
@@ -1747,7 +1743,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("tr - %s\n", version_string);
       exit (0);
     }
 
@@ -1761,11 +1757,14 @@ main (argc, argv)
 
   /* Change this test if it is legal to give tr no options and
      no args at all.  POSIX doesn't specifically say anything
-     either way, but it looks like they implied it's illegal
+     either way, but it looks like they implied it's invalid
      by omission.  If you want to make tr do a slow imitation
      of `cat' use `tr a a'.  */
   if (non_option_args > 2)
-    usage (2);
+    {
+      error (0, 0, "too many arguments");
+      usage (2);
+    }
 
   if (!delete && !squeeze_repeats && non_option_args != 2)
     error (1, 0, "two strings must be given when translating");
@@ -1785,9 +1784,13 @@ deleting and squeezing repeats");
 	--non_option_args;
       else
 	error (1, 0,
-	       "only one string may be given when deleting without squeezing repeats");
+	       "only one string may be given when deleting \
+without squeezing repeats");
     }
 
+  if (squeeze_repeats && non_option_args == 0)
+    error (1, 0, "at least one string must be given when squeezing repeats");
+
   spec_init (s1);
   if (parse_str ((unsigned char *) argv[optind], s1))
     exit (1);
@@ -1847,7 +1850,7 @@ deleting and squeezing repeats");
 		  if (ch == -1)
 		    {
 		      /* This will happen when tr is invoked like e.g.
-			 tr -cs A-Za-z0-9 '\012'.  */
+		         tr -cs A-Za-z0-9 '\012'.  */
 		      break;
 		    }
 		  xlate[i] = ch;
