@@ -35,6 +35,8 @@
 
    David MacKenzie <djm@gnu.ai.mit.edu> */
 
+#include <config.h>
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
@@ -98,17 +100,17 @@ static int have_read_stdin;
 static int exit_status;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"tabs", required_argument, NULL, 't'},
   {"all", no_argument, NULL, 'a'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -136,7 +138,7 @@ main (argc, argv)
 	  break;
 
 	case '?':
-	  usage ();
+	  usage (1);
 	case 'a':
 	  convert_entire_line = 1;
 	  break;
@@ -156,14 +158,14 @@ main (argc, argv)
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("unexpand - %s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   add_tabstop (tabval);
 
@@ -243,7 +245,7 @@ validate_tabstops (tabs, entries)
 {
   int prev_tab = 0;
   int i;
-  
+
   for (i = 0; i < entries; i++)
     {
       if (tabs[i] == 0)
@@ -262,7 +264,7 @@ unexpand ()
 {
   FILE *fp;			/* Input stream. */
   int c;			/* Each input character. */
-  /* Index in `tab_list' of next tabstop: */  
+  /* Index in `tab_list' of next tabstop: */
   int tab_index = 0;		/* For calculating width of pending tabs. */
   int print_tab_index = 0;	/* For printing as many tabs as possible. */
   int column = 0;		/* Column on screen of next char. */
@@ -271,6 +273,9 @@ unexpand ()
   int pending = 0;		/* Pending columns of blanks. */
 
   fp = next_file ((FILE *) NULL);
+  if (fp == NULL)
+    return;
+
   for (;;)
     {
       c = getc (fp);
@@ -435,11 +440,29 @@ next_file (fp)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-tab1[,tab2[,...]]] [-t tab1[,tab2[,...]]] [-a]\n\
-       [--tabs=tab1[,tab2[,...]]] [--all] [--help] [--version] [file...]\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -a, --all           convert all whitespace, instead of initial whitespace\n\
+  -t, --tabs=NUMBER   have tabs NUMBER characters apart instead of 8\n\
+  -t, --tabs=LIST     use comma separated list of explicit tab positions\n\
+      --help          display this help and exit\n\
+      --version       output version information and exit\n\
+\n\
+Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.  With\n\
+no FILE, or when FILE is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
