@@ -18,16 +18,7 @@
 /* Written by Paul Rubin, phr@ocf.berkeley.edu
    and David MacKenzie, djm@gnu.ai.mit.edu. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
 
 #include <stdio.h>
 #include <getopt.h>
@@ -39,6 +30,7 @@
 #define BUFFER_SIZE (16 * 1024)
 
 void error ();
+int safe_read ();
 
 static void wc ();
 static void wc_file ();
@@ -141,7 +133,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("wc - %s\n", version_string);
       exit (0);
     }
 
@@ -213,14 +205,32 @@ wc (fd, file)
 
   lines = words = chars = 0;
 
-  if (print_chars && !print_words && !print_lines
-      && fstat (fd, &stats) == 0 && S_ISREG (stats.st_mode))
+  /* When counting only bytes, save some line- and word-counting
+     overhead.  If FD is a `regular' Unix file, using fstat is enough
+     to get its size in bytes.  Otherwise, read blocks of BUFFER_SIZE
+     bytes at a time until EOF.  */
+  if (print_chars && !print_words && !print_lines)
+    {
+      if (fstat (fd, &stats) == 0 && S_ISREG (stats.st_mode))
     {
       chars = stats.st_size;
+	}
+      else
+	{
+	  while ((bytes_read = safe_read (fd, buf, BUFFER_SIZE)) > 0)
+	    {
+	      chars += bytes_read;
+	    }
+	  if (bytes_read < 0)
+	    {
+	      error (0, errno, "%s", file);
+	      exit_status = 1;
+	    }
+	}
     }
   else
     {
-      while ((bytes_read = read (fd, buf, BUFFER_SIZE)) > 0)
+      while ((bytes_read = safe_read (fd, buf, BUFFER_SIZE)) > 0)
 	{
 	  register char *p = buf;
 
